# Makefile for NT filesystem kernel module and tools.
#
# You should have the running kernel properly configured in
# /usr/src/linux to be sure module versioning is detected.

NTFS_DEVICE=	/dev/hdb7
MOUNT_POINT=	/zip
DEBUG_NTFS=	y
SMP_KERNEL=	n
GREP_DISPLAY=	512

MOUNT_PROG=	mount
UMOUNT_PROG=	umount
INSMOD_PROG=	/sbin/insmod
RMMOD_PROG=	/sbin/rmmod

################### End of config section ################

# Set up gcc options somewhat like the kernel Makefiles would.
WARNFLAG=	-Wall -Wstrict-prototypes
BUGFLAG=	-g3 -O
OPTFLAG=	-O2 -fomit-frame-pointer -m486 -malign-loops=2 -malign-jumps=2 -malign-functions=2
LDFLAGS=
MODFLAGS=	-D__KERNEL__ -DMODULE -D_POSIX_SOURCE

ifeq ($(DEBUG_NTFS),y)
CFLAGS := -pipe -fno-strength-reduce $(BUGFLAG) $(WARNFLAG)
#enable on demand
MODFLAGS := $(MODFLAGS) -DDEBUG
else
CFLAGS := -pipe -fno-strength-reduce $(OPTFLAG) $(WARNFLAG)
endif

# Find out if module versioning should be enabled.
ifeq (/usr/src/linux/.config,$(wildcard /usr/src/linux/.config))
include /usr/src/linux/.config
else
complain1:
	@echo You do not have the Linux kernel installed and
	@echo configured in /usr/src/linux. Compile it too.
endif
ifeq ($(CONFIG_MODVERSIONS),y)
MODFLAGS := $(MODFLAGS) -DCONFIG_MODVERSIONS
endif

# Check for _compiled_ kernel.
ifneq (/usr/src/linux/lib/lib.a,$(wildcard /usr/src/linux/lib/lib.a))
complain2:
	@echo You do not have the Linux kernel installed and
	@echo compiled in /usr/src/linux. Do that first.
endif

ifeq ($(SMP),y)
MODFLAGS := $(MODFLAGS) -D__SMP__
endif

COFLAGS= -l
KVERSION=$(shell uname -r)


SRCS= dump.c dir.c inode.c ntgrep.c ntcat.c ntdir.c ntdump.c ntfstools.c \
	ntfs_fs.c super.c ntchange.c ntfs.h util.c Makefile \
	mount_ntfs.c 44vfs/Makefile 44vfs/bsd.ntfs.mk 44vfs/ntfs_vfsops.c \
	44vfs/ntfs_vnops.c bsd.mk attr.c linux.mk ntfstypes.h version.h
ALLFILES= $(SRCS) README PROBLEMS ntfs.lpm doc \
	COPYING INSTALL INSTALL.linux patch
PROGRAMS= ntdir ntcat ntdump ntgrep ntchange

# Create config.h if it does not exist.
ifneq (config.h,$(wildcard config.h))
have_no_config:	config_h linux
endif

all:	linux

config_h:
	@echo '#define NTFS_VOLUME "'$(NTFS_DEVICE)'"' > config.h
	@echo '#define GREP_DISPLAY_SIZE '$(GREP_DISPLAY) >> config.h
	@echo '#define HAVE_GETOPT_LONG' >> config.h

vars:
	@echo MODFLAGS is $(MODFLAGS)
	@echo CFLAGS is $(CFLAGS)

linux:	$(PROGRAMS) ntfs.o

ntdump:	dump.o inode.o ntdump.o ntfstools.o dir.o super.o attr.o util.o
	$(CC) -o $@ $^

ntgrep: dump.o inode.o ntgrep.o ntfstools.o dir.o super.o attr.o util.o
	$(CC) -o $@ $^

ntdir:	dir.o inode.o ntdir.o ntfstools.o super.o attr.o util.o
	$(CC) -o $@ $^

ntcat:	dir.o inode.o ntcat.o ntfstools.o super.o attr.o util.o
	$(CC) -o $@ $^

ntchange:	ntchange.o dir.o inode.o ntfstools.o super.o attr.o util.o
	$(CC) -o $@ $^

mount_ntfs:	mount_ntfs.c getmntopts.o
	$(CC) -o $@ $^

getmntopts.o:	$(MOUNT_PROG)/getmntopts.c
	$(CC) -c $^

ntfs_fs.o:	ntfs_fs.c
	$(CC) $(CFLAGS) $(MODFLAGS) -c ntfs_fs.c

ntfs.o:	ntfs_fs.o inode.o dir.o super.o attr.o util.o
	$(LD) -r $^ /usr/src/linux/lib/lib.a -o ntfs.o

ntfs2.o: ntfs_fs.c inode.o dir.o super.o
	$(CC) $(CFLAGS) $(MODFLAGS) -DSECOND -c ntfs_fs.c -o ntfs_fs2.o 
	$(LD) -r ntfs_fs2.o inode.o dir.o super.o -o ntfs2.o

clean:
	rm -f core config.h doc/*~ doc/DEADJOE DEADJOE *~ *.o $(PROGRAMS)

install:	all
	install -d /usr/local/bin /lib/modules/$(KVERSION)/fs
	install $(PROGRAMS) /usr/local/bin
	install ntfs.o /lib/modules/$(KVERSION)/fs
	depmod -a

co:	$(SRCS)

ci:	$(SRCS)
	ci -l $(SRCS)

mount:	ntfs.o
	sync
	$(INSMOD_PROG) ntfs.o ntdebug=1
	$(MOUNT_PROG) -t ntfs -oumask=0 $(NTFS_DEVICE) $(MOUNT_POINT)

umount:	
	$(UMOUNT_PROG) $(MOUNT_POINT)
	$(RMMOD_PROG) ntfs

ntfs.tar.gz: $(ALLFILES)
	rm -f ntfs.tar.gz
	date +'#define NTFS_VERSION "%y%m%d"' >version.h
	tar -C .. -cf ntfs.tar $(ALLFILES:%=ntfs/%)
	gzip -9 ntfs.tar

always:

# Q: Who knows to generate a portable makedepend output?
# A: Linus Torvalds
attr.o: attr.c ntfs.h ntfstypes.h
dir.o: dir.c ntfs.h ntfstypes.h config.h
dump.o: dump.c config.h ntfs.h ntfstypes.h
inode.o: inode.c ntfs.h ntfstypes.h config.h
my_create.o: my_create.c ntfs.h ntfstypes.h
ntcat.o: ntcat.c config.h ntfs.h ntfstypes.h
ntchange.o: ntchange.c config.h ntfs.h ntfstypes.h
ntdir.o: ntdir.c config.h ntfs.h ntfstypes.h version.h
ntdump.o: ntdump.c config.h ntfs.h ntfstypes.h version.h
ntfs_fs.o: ntfs_fs.c config.h ntfs.h ntfstypes.h
ntfstools.o: ntfstools.c config.h ntfs.h ntfstypes.h
ntgrep.o: ntgrep.c config.h ntfs.h ntfstypes.h
super.o: super.c ntfs.h ntfstypes.h config.h
